
import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import './dialog_state_mixin.dart';

typedef DialogActionType = Future<dynamic> Function();

//  每个弹窗具体信息描述
class DialogAction {
  DialogAction({
    this.priorityLevel = 0,
    this.action
  });

  //  优先等级
  int priorityLevel;
  //  具体的弹屏行为
  DialogActionType action;
}

//  页面对应的弹窗信息，当前是否有弹窗正在显示中、页面对应的要显示的弹窗数组
class DialogDetail {
  DialogDetail({
    this.dialogActions
  });

  //  当前是否有弹窗正在显示中
  bool isShowingDialog = false;
  //  路由页面上的弹屏数组
  List<DialogAction> dialogActions;
}

class DialogManage {
  DialogManage();

  //  页面路由与对应的弹窗对应关系
  Map<ModalRoute, DialogDetail> _routeDialogMap = {};

  //  订阅弹窗显示状态变化的队列
//  Set<Map<ModalRoute, DialogStateMixin>> _dialogStateMixinSet = Set();
  Map<ModalRoute, List<DialogStateMixin>> _dialogStateMixinSet = {};


  //  添加弹窗展示状态变化监听
  void addListener (DialogStateMixin dialogStateMixin, BuildContext context) {
    ModalRoute route = ModalRoute.of(context);
    if (_dialogStateMixinSet[route] == null) {
      _dialogStateMixinSet[route] = [];
    }
    _dialogStateMixinSet[route].add(dialogStateMixin);
  }

  //  移除弹窗展示状态变化监听
  void removeListener (DialogStateMixin dialogStateMixin, BuildContext context) {
    ModalRoute route = ModalRoute.of(context);
    if (_dialogStateMixinSet[route] != null) {
      for (int i = 0, length = _dialogStateMixinSet[route].length; i < length; i++) {
        DialogStateMixin item = _dialogStateMixinSet[route][i];
        if (item == dialogStateMixin) {
          _dialogStateMixinSet[route].remove(dialogStateMixin);
        }
      }
    }
  }

  //  获取指定页面的弹窗显示状态
  bool getIsShowingState (BuildContext context) {
    ModalRoute route = ModalRoute.of(context);
    bool isShowing = false;
    if (_routeDialogMap[route] != null) {
      isShowing = _routeDialogMap[route].isShowingDialog;
    }
    return isShowing;
  }

  //  flutter showDialog包装
  Future<dynamic> showFlutterDialog ({
    @required BuildContext context,
    bool barrierDismissible = true,
    WidgetBuilder builder,
    int priorityLevel
  }) {
    DialogActionType fn = () {
      return showDialog(
        context: context,
        barrierDismissible: barrierDismissible,
        builder: builder
      );
    };
    return requestShowDialog(context, fn, priorityLevel: priorityLevel);
  }

  //  请求弹窗
  Future<dynamic> requestShowDialog (BuildContext context, DialogActionType fn, { int priorityLevel = 0 }) async {
    ModalRoute route = ModalRoute.of(context);
    DialogAction dialogAction = DialogAction(priorityLevel: priorityLevel, action: fn);

    if (_routeDialogMap[route] != null) {
      if (_routeDialogMap[route].isShowingDialog) {
        return Future.value('此页面已经有弹窗正在显示中，弹窗显示失败');
      } else {
        _routeDialogMap[route].dialogActions.add(dialogAction);
      }
    } else {
      _routeDialogMap[route] = DialogDetail(dialogActions: [dialogAction]);
    }

    await Future.delayed(Duration.zero);

    int targetIndex = _getHighestPriorityIndex(_routeDialogMap[route].dialogActions);

    if (_routeDialogMap[route].dialogActions[targetIndex] == dialogAction) {
      _routeDialogMap[route].isShowingDialog = true;
      _sendDialogShowChangeEvent(route, true);
      Future<dynamic> futureResult = _routeDialogMap[route].dialogActions[targetIndex].action();
      futureResult.then((_) {
        _routeDialogMap[route].isShowingDialog = false;
        _sendDialogShowChangeEvent(route, false);
        _deleteTargetDialogAction(route, dialogAction);
      });
      return futureResult;
    } else {
      _deleteTargetDialogAction(route, dialogAction);
      return Future.value('优先级不够，弹窗显示失败');
    }
  }

  //  给指定页面的监听者发送弹窗状态变化事件
  void _sendDialogShowChangeEvent (ModalRoute route, bool isShowingDialog) {
    _dialogStateMixinSet.forEach((ModalRoute key, List<DialogStateMixin> value) {
      if (key == route) {
        value.forEach((DialogStateMixin item) {
          if (isShowingDialog) {
            item.onDialogShow();
          } else {
            item.onDialogHide();
          }
        });
      }
    });
  }

  //  删除指定的弹窗行为
  void _deleteTargetDialogAction (ModalRoute route, DialogAction dialogAction) {
    for (int i = 0, length = _routeDialogMap[route].dialogActions.length; i < length; i++) {
      if (_routeDialogMap[route].dialogActions[i] == dialogAction) {
        _routeDialogMap[route].dialogActions.removeAt(i);
        break;
      }
    }
  }

  //  取出优先级最高的那个弹窗位置
  int _getHighestPriorityIndex (List<DialogAction> dialogList) {
    //  只有一个弹窗，取第一个弹窗执行，多个弹窗取出优先级最高的弹窗执行
    if (dialogList.length == 1) {
      return 0;
    } else {
      int targetIndex = 0;
      int targetPriorityLevel = dialogList[0].priorityLevel;
      for (int i = 0, length = dialogList.length; i < length; i++) {
        int currentPriorityLevel = dialogList[i].priorityLevel;
        if (currentPriorityLevel < targetPriorityLevel) {
          targetIndex = i;
          targetPriorityLevel = currentPriorityLevel;
        }
      }
      return targetIndex;
    }
  }
}




